@@ -249,6 +249,19 @@ isDebug <- function()
     return (.lkdefaults$debug)
 }
 
+labkey.setWafEncoding <- function(wafEncode=TRUE)
+{
+    .lkdefaults$wafEncode = wafEncode;
+}
+
+isWafEncoding <- function()
+{
+    if (is.null(.lkdefaults$wafEncode)) {
+        return (TRUE)
+    }
+    return (.lkdefaults$wafEncode)
+}
+
 
 isRequestError <- function(response, status_code) 
 {
@@ -305,4 +318,34 @@ verboseOutput <- function(title, content)
         print(content)
         print(paste("*******************END ",title,"*********************", sep=""))
     }
+}
+
+# Obfuscates content that's often intercepted by web application firewalls that are scanning for likely
+# SQL or script injection. We have a handful of endpoints that intentionally accept SQL or script, so we
+# encode the text to avoid tripping alarms. It's a simple BASE64 encoding that obscures the content, and lets the
+# WAF scan for and reject malicious content on all other parameters. See Issue 48509.
+wafEncode <- function(value)
+{
+    if (!is.null(value) && is.character(value))
+    {
+        value <- encodeURIComponent(value)
+        value <- base64_enc(value)
+        value <- gsub("\n", "", value) # jsonlite:base64_enc concats long strings with \n char
+        value <- paste0("/*{{base64/x-www-form-urlencoded/wafText}}*/", value)
+    }
+    return (value)
+}
+
+# URL Encode string.
+# NOTE: made to match JavaScript encodeURIComponent()
+encodeURIComponent <- function(value)
+{
+    value <- URLencode(value, reserved = TRUE)
+    # The following characters need to be decoded to match server side behavior: !  * ' ( )
+    value <- gsub("%21", "!", value)
+    value <- gsub("%2A", "*", value)
+    value <- gsub("%27", "'", value)
+    value <- gsub("%28", "(", value)
+    value <- gsub("%29", ")", value)
+    return (value)
 }
\ No newline at end of file
